<?php
// $Id$

/**
 * Admin functions for Theme Hues
 */

/**
 * 
 * Theme Hues configuration form for the admin user.
 */
function theme_hues_admin() {
  module_load_include('inc', 'theme_hues', 'theme_hues.inc');
  
  $form = array('#attributes' => array('enctype' => "multipart/form-data"));
  _theme_hues_admin_settings($form);
  
  $rulesets = theme_hues_rulesets();
  foreach ($rulesets as $ruleset) {
    _theme_hues_admin_ruleset_fieldset($form, $ruleset);
  }
      
  // add one empty ruleset
  $dummy = _theme_hues_empty_ruleset();
  _theme_hues_admin_ruleset_fieldset($form, $dummy);

  $form['submit'] = array(
    '#type'	      => 'submit',
    '#value'	    => t("Save and rebuild CSS files"),
    '#attributes'	=> array('id' => 'save')
  );
  $form['clear'] = array(
    '#type'	      => 'submit',
    '#value'	    => t("Clear all rules and delete all CSS files"),
    '#attributes'	=> array('id' => 'clear')
  );
  
  return $form;
}

/**
 * 
 * Validator for the Theme Hues configuration form for the admin user.
 * 
 * Settings:
 *   - check that the image [and font - TODO] directories are valid
 *   
 * Rulesets:
 * 	Until we move creating a ruleset into it's own form, we even have to check 
 * 	required fields here.
 *   - Name
 *   		- required
 *   - Selectors
 *   		- required
 *   		- @todo conforms to CSS standards
 *   
 *   - Property
 *      - required
 *      - @todo is a valid CSS property 
 *      
 *   - Restrictions
 *      - user_all: check that they HAVEN'T selected a role
 *   	  - user_uid: check that they HAVEN'T selected a role
 *      - user_role: check that they HAVE selected a role
 *      
 *      - node_all: check that they HAVEN'T selected a node type
 *      - node_nid: check that they HAVEN'T selected a node type
 *      - node_type: check that they HAVE selected a node type
 */
function theme_hues_admin_validate(&$form, &$form_state) {
  if ($form_state['clicked_button']['#attributes']['id'] == 'clear') {
    // they are clearing everything, so don't bother validating
    return;
  }
  
  file_prepare_directory($form_state['values']['image_path'], TRUE, 'image_path');
  /*
  foreach (array('font_path', 'image_path') as $dirpath) {
    file_check_directory($form_state['values'][$dirpath], TRUE, $dirpath);
  }
  */

  $rulesets = _theme_hues_admin_collect_form($form_state['values']);
  foreach ($rulesets as $ruleset) {
    if (theme_hues_ruleset_is_empty($ruleset)) {
      continue;
    }
    
    $field = 'ruleset-' . $ruleset['rid'];
    if (!$ruleset['name']) {
      form_set_error($field . "-name", t("You must give this ruleset a user friendly name."));
    }
    
    if ($ruleset['restrictions']['global']['value']) {
      // global settings apply to everyone, so don't bother checking
      // the rest of the restrictions
      continue;
    }
        
    $user_cond = $ruleset['restrictions']['user']['condition'];
    $user_val  = $ruleset['restrictions']['user']['value'];
    if (($user_cond == 'uid' || $user_cond == 'all') && ($user_val && is_array($user_val))) {
      form_set_error($field . '-restrictions-user', t("You must specify the user roles restriction if you have selected user roles."));      
    }
    if ($user_cond == 'role' && (!$user_val || !is_array($user_val))) {
      form_set_error($field . '-restrictions-user', t("You must specify some user roles if you choose the user roles restriction."));      
    }
    
    $node_cond = $ruleset['restrictions']['node']['condition'];
    $node_val  = $ruleset['restrictions']['node']['value'];
    if (($node_cond == 'none' || $node_cond == 'nid') && ($node_val && is_array($node_val))) {
      form_set_error($field . '-restrictions-node', t("You must specify the node type restriction if you have selected node types."));      
    }
    if ($node_cond == 'type' && (!$node_val || !is_array($node_val))) {
      form_set_error($field . '-restrictions-node-type', t("You must specify the some node types if you choose the node type restriction."));      
    }
    if ($node_cond == 'nid' && !$node_val) {
      form_set_error($field . '-restrictions-node-nid', t("You must specify one or more node ids if you choose the specific node restriction."));      
    }
    if ($node_cond == 'none' && $node_val) {
      form_set_error($field . '-restrictions-node-nid', t("You cannot specify for no node restrictions, and then specify conditions like node types or node ids."));      
    }
  }
}

/**
 * 
 * Submit handler for the Theme Hues configuration form for the admin user.
 */
function theme_hues_admin_submit($form, &$form_state) {
  module_load_include('inc', 'theme_hues', 'theme_hues.css');
  if (!$form_state['values']['enabled']) {
    theme_hues_css_clear_files();
    drupal_clear_css_cache();
    variable_set('theme_hues_active', FALSE);
    drupal_set_message(t("Theme Hues CSS files have been disabled."));
  }
  else {
    $old = variable_get('theme_hues_active', TRUE);
    if (!$old) {
      drupal_set_message(t("Theme Hues CSS files have been enabled."));
    }
    
    variable_set('theme_hues_active', TRUE);
  }
  
  variable_set('theme_hues_css_path', $form_state['values']['css_path']);
  variable_set('theme_hues_image_path', $form_state['values']['image_path']);
  //variable_set('theme_hues_font_path', $form_state['values']['font_path']);
  
  $rulesets = _theme_hues_admin_collect_form($form_state['values']);
  foreach ($rulesets as $ruleset) {
    if (!$ruleset['delete']) {
      theme_hues_save_ruleset($ruleset);
    }
    else {
      theme_hues_delete_ruleset($ruleset);
    }
  }
  
  theme_hues_css_generate_files();
}

/**
 * 
 * Iterate through the form values, collecting the data regarding rulesets and putting it into 
 * a traversable data structure.
 * 
 * @param $values
 * 
 * @return
 * An array of rulesets (no validation done here!) that looks like:
 * 
 * array(
 *   $rid => array(
 *   	'rid' 				=> $rid
 *   	'name'			  => $name,
 *   	'description'	=> $description,
 *    'delete'			=> TRUE|FALSE,
 *    'selectors'		=> array(
 *    	$sid => array(
 *   		'sid'				  => $sid,
 *   		'selectors'   => $selectors, 
 *      'property'	  => $property,
 *    	'important'		=> TRUE|FALSE,
 *    	'user_files'	=> TRUE|FALSE,
 *    	),
 *    	...
 *    ),
 *    'restrictions' => array(
 *    	$context => array(
 *    		'context' 	=> $context,
 *    		'value'			=> $value,
 *    		'condition'	=> $condition
 *      ),
 *      ...
 *    )
 *  )
 *
 */
function _theme_hues_admin_collect_form($values) {
  $rulesets = array();
  foreach ($values as $key => $value) {
    $fields = explode('-', $key);
    if ($fields[0] != 'ruleset') {
      continue;
    }
    
    $rid = $fields[1];
    if (!(int) $rid && $rid != 'new') {
      continue;
    }
    
    if (!isset($rulesets[$rid])) {
      $rulesets[$rid] = array(
        'rid'	          => $rid,
        'name'	        => '',
        'description'   => '',
        'delete'			  => FALSE,
        'selectors'			=> array(),
        'restrictions'	=> array()
      );
    }
    
    // name, description, delete
    if (count($fields) == 3) {
      $rulesets[$rid][$fields[2]] = $value;
      continue;
    }
        
    if ($fields[2] == 'selectors') {
      $sid = $fields[3];
      
      if (!isset($rulesets[$rid]['selectors'][$sid])) {
        $rulesets[$rid]['selectors'][$sid] = array(
          'sid'				 => $sid,
          'selectors'  => '',
          'property'	 => '',
          'files'			 => array(),
          'important'	 => FALSE,
          'user_files' => FALSE
        );
      }
      
      if ($fields[4] == 'files') {
        if ($fields[5] == 'delete') {
          foreach ($value as $fid => $checked) {
            if (!$checked) {
              continue;
            }
            
            $rulesets[$rid]['selectors'][$sid]['files'][$fid] = array(
              'fid'	     => $fid,
              'delete'	 => TRUE
            ); 
          }
        }
        else {
          // new file
          // uploading font files is not implemented in the UI, so this should only ever resolve to 'image'
          $path = (strstr($fields[6], 'image'))? 'image' : 'font';
          $fn = 'theme_hues_' . $path . '_path';
          $dest = $fn();
          
          $file = file_save_upload($key, array(), "public://" . $dest, FILE_EXISTS_REPLACE);
          if ($file) {
            if ($path == 'image' && file_validate_extensions($file, 'png jpg gif')) {
              drupal_set_message(t("You can only upload an image file for this property.  !filename does not look like an image to me.", array('!filename' => $file->filename)), 'error');
            }
            if ($path == 'font' && file_validate_extensions($file, 'ttf otf fnt fon')) {
              drupal_set_message(t("You can only upload a font file for this property.  !filename does not look like a font file to me.", array('!filename' => $file->filename)), 'error');
            }
            else {
              $file->status = FILE_STATUS_PERMANENT;
              file_save($file);
              
              $rulesets[$rid]['selectors'][$sid]['files'][$file->fid] = array(
                'fid'     => $file->fid,
                'global'	=> TRUE,
                'file'		=> $file
              ); 
            }
          }
        }
      }
      else {
        $rulesets[$rid]['selectors'][$sid][$fields[4]] = $value;
      }      
      continue;
    }
    
    if ($fields[2] == 'restrictions') {
      $context = $fields[3];
      
      if (!isset($rulesets[$rid]['restrictions'][$context])) {
        $rulesets[$rid]['restrictions'][$context] = array(
          'context'   => $context, 
          'condition' => NULL,  
          'value'     => NULL
        );
      }
      
      if ($context == 'user' || $context == 'node') {
        if (count($fields) == 4) {
          // ruleset-RID-restrictions-user || ruleset-RID-restrictions-node
           $rulesets[$rid]['restrictions'][$context]['condition'] = $value;
           continue;
        }
        else {
          if (is_array($value)) {
            // ruleset-RID-restrictions-user-role || ... node-type
            $active = array();
            foreach ($value as $id => $checked) {
              if ($checked) {
                $active[] = $id;
              }
            }
            
            if ($active) {
              $rulesets[$rid]['restrictions'][$context]['value'] = $active;
            }
            continue;
          }
          else {
            // ... node-nid
            if ($value) {
              $rulesets[$rid]['restrictions'][$context]['value'] = $value;
            }
          }
        }
      }
      else {
        $rulesets[$rid]['restrictions'][$context]['value']     = $value;
        $rulesets[$rid]['restrictions'][$context]['condition'] = (isset($value) && $value);
      }
    }
  }
  
  return $rulesets;
}

/**
 * 
 * Add general settings frameset to our admin form
 * 	- global kill switch
 *  - image path
 *  - font path
 *  
 * @param &$form
 */
function _theme_hues_admin_settings(&$form) {
  $form['settings'] = array(
    '#type'		      => 'fieldset',
    '#title'	      => t("Theme Hues general settings"),
    '#collapsed'	  => FALSE,
    '#collapsible'  => FALSE,
    'enabled'	=> array(
      '#type'		        => 'checkbox',
      '#title'	        => t("Enable Theme Hues?"),
      '#description'	  => t("Uncheck this option to remove Theme Hues css stylesheets temporarily.  No settings will be deleted."),
      '#default_value'	=> variable_get("theme_hues_enabled", TRUE)
    ),
    'css_path'	=> array(
      '#type'		        => 'textfield',
      '#title'	        => t("Theme Hues CSS directory"),
      '#description'	  => t("The path to store Theme Hues CSS files in.  This path will be created for you under your public files directory."),
      '#default_value'	=> variable_get("theme_hues_css_path", "theme_hues_css"),
      '#required'				=> TRUE
    ),
    'image_path'	=> array(
      '#type'		        => 'textfield',
      '#title'	        => t("Theme Hues image directory"),
      '#description'	  => t("The path to store Theme Hues images in.  This path will be created for you under your public files directory."),
      '#default_value'	=> variable_get("theme_hues_image_path", "theme_hues_images"),
      '#required'				=> TRUE
    )/*,
    'font_path'	=> array(
      '#type'		        => 'textfield',
      '#title'	        => t("Theme Hues font directory"),
      '#description'	  => t("The path to store Theme Hues fonts in.  This path will be created for you under your public files directory."),
      '#required'				=> TRUE
    )*/
  );
}

/**
 * 
 * Add a ruleset fieldset
 * 
 * @param &$form
 * 
 * @param $ruleset
 */
function _theme_hues_admin_ruleset_fieldset(&$form, $ruleset) {
  $key = "ruleset-" . $ruleset['rid'];

  $form[$key] = array(
    '#type'	        => 'fieldset',
    '#title'		    => ($ruleset['rid'] == 'new')? t("Add a new ruleset") : $ruleset['name'],
    '#collapsible'	=> TRUE,
    '#collapsed'		=> TRUE
  );
  
  $k_name       = "ruleset-" . $ruleset['rid'] . "-name";
  $k_desc       = "ruleset-" . $ruleset['rid'] . "-description";
  $k_delete     = "ruleset-" . $ruleset['rid'] . "-delete";
  
  $k_selectors  = "ruleset-" . $ruleset['rid'] . "-selectors";
  $k_restrict   = "ruleset-" . $ruleset['rid'] . "-restrictions";
  
  $form[$key][$k_delete] = array(
    '#type'			      => 'checkbox',
    '#title'		      => t("Delete?"),
    '#description'		=> t("Check this to delete this ruleset on submit.")
  );
  $form[$key][$k_name] = array(
    '#type'			      => 'textfield',
    '#title'		      => t("Name"),
    '#default_value'	=> $ruleset['name'],
    '#description'		=> t("Give a descriptive name for this ruleset.  It will be displayed on the form the end user uses to set the value for the ruleset properties.")
  );
  $form[$key][$k_desc] = array(
    '#type'			      => 'textarea',
    '#title'		      => t("Description"),
    '#default_value'	=> $ruleset['description'],
    '#description'		=> t("Insert a longer description for this ruleset.  You may want to explain here exactly how this ruleset will be applied on the site, for instance.")
  );
  
  $form[$key][$k_selectors] = array(
    '#type'		      => 'fieldset',
    '#title'	      => t("Selectors"),
    '#collapsible'	=> TRUE,
    '#collapsed'		=> TRUE,
    '#description'	=> t("Insert all of the selectors associated with this ruleset.  Save the form to add more selectors."),
  );
  
  $k_selector = "ruleset-" . $ruleset['rid'] . '-selector-';
  foreach ($ruleset['selectors'] as $selector) {
    $form[$key][$k_selectors][$k_selector . $selector['sid']] = _theme_hues_admin_selector_fieldset($selector);
  }
  
  $dummy = 0;
  while($dummy < 3) {
    $new_selector = array(
      'rid'					=> $ruleset['rid'],
      'sid'	        => "new$dummy",
      'selectors'   => "",
      'property'	  => "",
      'important'	  => FALSE,
      'user_files'	=> FALSE
    );
    
    $form[$key][$k_selectors][$k_selector . $new_selector['sid']] = _theme_hues_admin_selector_fieldset($new_selector);
    $dummy++;
  }
  
  $form[$key][$k_restrict] = array(
    '#type'			    => 'fieldset',
    '#title'		    => t("Restrictions"),
    '#collapsed'	  => TRUE,
    '#collapsible'	=> TRUE,
  );
  
  $k_restriction = "ruleset-" . $ruleset['rid'] . "-restrictions-";
  foreach ($ruleset['restrictions'] as $context => $restriction) {
    if (!isset($restriction['rid'])) {
      // for new rulesets, which won't have rid set.
      $restriction['rid'] = $ruleset['rid'];
    }
    
    $k_context = $k_restriction . $restriction['context'];
    $fn = '_theme_hues_admin_element_' . $restriction['context'];
    
    if (function_exists($fn)) {
      $form[$key][$k_restrict][$k_context] = $fn($restriction);
    }
  }
  
  //dpm($form);
}

function _theme_hues_admin_selector_fieldset($selector) {
  $k_base = 'ruleset-' . $selector['rid'] . '-selectors-' . $selector['sid'] . '-';
  
  $f_files = NULL;
  if ($selector['property'] && theme_hues_is_file_property($selector['property'])) {
    $f_files = _theme_hues_admin_file_fieldset($selector);
  }
  
  return array(
    '#type'		      => 'fieldset',
    '#title'				=> (isset($selector['selectors']) && $selector['selectors'])? $selector['selectors'] : t("new selector"),
    '#collapsible'	=> TRUE,
    '#collapsed'		=> FALSE,
    $k_base . 'selectors' => array(
      '#type'	         => 'textarea',
      '#default_value' => $selector['selectors'],
      '#title'				 => t("Selectors"),
      '#description'	 => t("Insert one CSS selector per line, ie. 'h2', '#foo' or '#wrapper #content .special div:first-child'")
    ),
    $k_base . 'property' => array(
      '#type'			      => 'textfield',
      '#title'		      => t("Property"),
      '#default_value'	=> $selector['property'],
      '#description'		=> t("Insert the CSS property, ie. 'background-color' or 'font-size'")
    ),
    $k_base . 'files' => $f_files,
    $k_base . 'important' => array(
      '#type'			      => 'checkbox',
      '#title'		      => t("Important?"),
      '#default_value'	=> $selector['important'],
      '#description'		=> t("Check this to add ' !important' to your property.  You probably actually want to make a more specific selector (ie. '#foo h2' rather than 'h2').")
    ),
    $k_base . 'user_files' => array(
      '#type'			      => 'checkbox',
      '#title'		      => t("Allow user files?"),
      '#default_value'	=> $selector['user_files'],
      '#description'		=> t("Check this to allow users to upload their own files (if this is an image property).")
    )
  );
}

function _theme_hues_admin_file_fieldset($selector) {
  $k_base = 'ruleset-' . $selector['rid'] . '-selectors-' . $selector['sid'] . '-';
  
  $fieldset = array(
    '#type'		      => 'fieldset',
    '#title'	      => t("!property files", array('!property' => $selector['property'])),
    '#collapsible'	=> FALSE
  );
  
  $files = theme_hues_files($selector);
  if ($files) {
    $options = array();
    foreach ($files as $file) {
      $options[$file['fid']] = $file['file']['filename'];
      
      $fieldset[$k_base . "files-delete"] = array(
        '#type'	        => 'checkboxes',
       	'#title'	      => t("Globally available"),
        '#options'	    => $options,
        '#description'	=> t("Images that are available to ALL users who can set this property.  Checked images will be deleted when you save this form.")
      );
    }
  }
  
  $fieldset[$k_base . 'files-new-' . str_replace("-", "_", $selector['property'])] = array(
    '#type'		      => 'file',
    '#title'	      => t("Upload a new global file"),
    '#description'	=> t("Upload a file which will be available to ALL users who can set this property.  You may also allow users to upload their own files by selecting the checkbox below.")
  );
  
  return $fieldset;
}

function _theme_hues_admin_element_global($restriction) {
  return array(
    '#title'		      => t("Set globally"),
    '#type'	          => 'checkbox',
    '#default_value'	=> $restriction['condition'],
    '#description' 		=> t("Make this a global rule, visible to all users in any context.  This overrides any other restrictions for this ruleset.")
  );
}

function _theme_hues_admin_element_user($restriction) {
  $roles  = user_roles();
  $k_base = 'ruleset-' . $restriction['rid'] . '-restrictions-';
  
  return array(
    '#type'			    => 'fieldset',
    '#title'		    => t("User restrictions"),
    '#collapsible'	=> FALSE,
    $k_base . "user" => array(
      '#title'		=> t("Specify the user"),
      '#type'	    => 'radios',
      '#options'	=> array(
      	'uid'     => t("Only the user who created the customization will see them (personalization)"),
        'role'    => t("Only users of the selected roles below will see the customizations, no matter who makes them."),
        'content' => t("All users will see any user's customizations, but only on content created by that user."),
    		'all'		  => t("All users will see the customizations.")
        ),
      '#default_value'	=> $restriction['condition'],
      '#description' 		=> t("Select which users will see eventual custom settings.")
    ),
    $k_base . "user-role" => array(
      '#title'		     => t("Roles"),
      '#type'					 => 'checkboxes',
    	'#options'       => $roles,
      '#default_value'	=> ($restriction['value'])? $restriction['value'] : array(),
      '#description' 	 => t("If you selected roles above, check the roles that will see custom settings.")
    )
  );
}

function _theme_hues_admin_element_node($restriction) {
  $types   = array();
  $bundles = field_info_bundles('node');
  foreach ($bundles as $type => $info) {
    $types[$type] = $info['label'];
  }
  $k_base = 'ruleset-' . $restriction['rid'] . '-restrictions-';
  
  $default_value = $restriction['value'];
  if (is_array($restriction['value']) && $restriction['condition'] == 'type') {
    $default_value = array();
    foreach ($restriction['value'] as $type) {
      $default_value[$type] = $type;
    }
  }
  if (is_array($restriction['value']) && $restriction['condition'] == 'nid') {
    $default_value = join(", ", $restriction['value']);
  }
  return array(
    '#type'			    => 'fieldset',
    '#title'		    => t("Node restrictions"),
    '#collapsible'	=> FALSE,
    $k_base . "node" => array(
      '#title'		=> t("Specify the node restriction"),
      '#type'	    => 'radios',
      '#options'	=> array(
      	'type'  => t("Limit custom settings to nodes of a specific type."),
        'nid'   => t("Limit custom settings to specific nodes, regardless of type."),
        'none'  => t("Do not limit for any node.")
        ),
      '#default_value'	=> $restriction['condition'],
      '#description'    => t("Select which nodes custom settings will apply to.")
    ),
    $k_base . "node-type" => array(
      '#title'					=> t("Specify the node type"),
      '#type'	          => 'checkboxes',
      '#options'	      => $types,
      '#default_value'	=> ($restriction['condition'] == 'type')? $default_value : array(),
      '#description'		=> t("If you chose to limit settings by type, select which types settings will be visible on.")
    ),
    $k_base . "node-nid" => array(
      '#type'	          => 'textfield',
      '#title'					=> t("Node nid"),
      '#default_value'	=> ($restriction['condition'] == 'nid')? $default_value : "",
      '#description'		=> t("If you chose to limit settings to individual nodes, insert the node nids here in a comma delimited list.")
    )
  );
}

function _theme_hues_admin_element_path($restriction) {
  return array(
    '#title'					=> t("Restrict to path(s)"),
    '#type'	          => 'textarea',
    '#default_value'	=> $restriction['value'],
    '#description'		=> t("Limit this context to a set of paths.  You may use the '%' wildcard.  Insert one path per row.")
  );
}

function _theme_hues_admin_element_theme($restriction) {
  return array(
    '#title'					=> t("Restrict to theme(s)"),
    '#type'	          => 'textarea',
    '#default_value'	=> $restriction['value'],
    '#description'		=> t("Limit this context to a set of themes.  Insert one theme key per row.")
  );
}

function _theme_hues_admin_element_domain($restriction) {
  return array(
    '#title'					=> t("Restrict to domain(s)"),
    '#type'	          => 'textarea',
    '#default_value'	=> $restriction['value'],
    '#description'		=> t("Limit this context to a set of domains.  Insert one domain key per row.")
  );
}