<?php
// $Id$

/**
 * User functions for Theme Hues
 */

/**
 * 
 * Theme Hues configuration form for the end user.
 */
function theme_hues_configure() {
  drupal_add_js(drupal_get_path('module', 'theme_hues') . '/js/theme_hues.js');
  
  $form = array('#attributes' => array('enctype' => "multipart/form-data"));
  
  $rulesets = theme_hues_rulesets();
  if (!$rulesets) {
    drupal_set_message(t("There are no rulesets configured for Theme Hues."), 'warning');
    return $form;
  }
  
  foreach ($rulesets as $ruleset) {
    _theme_hues_user_fieldset($form, $ruleset);
  }
  
  $form['save'] = array(
    '#type'		=> 'submit',
    '#value'	=> t("Save settings")
  );
  
  return $form;
}

/**
 * 
 * Generate the form elements for the given rule
 * 
 * @param &$form
 * 
 * @param $ruleset
 */
function _theme_hues_user_fieldset(&$form, $ruleset) {
  module_load_include('inc', 'theme_hues', 'theme_hues.css');
  $k_base = 'ruleset-' . $ruleset['rid'];
  
   $form[$k_base] = array(
     '#type'		    => 'fieldset',
     '#title'		    => $ruleset['name'],
     '#description'	=> $ruleset['description']
   );
   
   foreach ($ruleset['selectors'] as $selector) {
     $k_selector = $k_base . '-selector-' . $selector['sid'];
     $k_value    = $k_selector . '-value';
     $form[$k_base][$k_selector]['js-' . $selector['sid']] = array(
       '#type'		    => 'hidden',
       '#value'		    => theme_hues_css_selector($selector),
       '#attributes'	=> array('id' => 'selector-' . $selector['sid'], 'rel' => 'selector')
     );
     
     $form[$k_base][$k_selector][$k_value] = _theme_hues_user_field($selector);
   }
}

/**
 * 
 * Get a field for setting an instance property value
 * 
 * @param $selector
 * 
 * @return
 * A form element
 */
function _theme_hues_user_field($selector) {
  if (strstr($selector['property'], 'image')) {
    return _theme_hues_user_file_field('image', $selector);
  }
  /*
  if ($selector['property'] == 'font-family') {
    return _theme_hues_user_file_field('font', $selector);
  }
  */
  
  $type       = 'textfield';
  $attributes = array('id' => 'property-' . $selector['sid'], 'rel' => check_plain($selector['property']));  
  if (strstr($selector['property'], 'color') && module_exists('jquery_colorpicker')) {
    $type = 'colorpicker';
  }
  
  $value = theme_hues_css_value($selector);
  
  return array(
    '#title'	        => $selector['property'],
    '#type'		        => $type,
    '#default_value'	=> $value,
    '#attributes'	    => $attributes
  );
}

/**
 * 
 * Get an image or font field for this selector.
 * 
 * If user files are allowed, presents a list of previously uploaded files by this user and globally available files,
 * plus a file upload field to add a new file.
 * 
 * Otherwise, just presents a list of pre-loaded files.
 * 
 * @param $property
 * 
 * @param $selector
 */
function _theme_hues_user_file_field($property, $selector) {
  $options = array('-1' => t("none"));
  $files = theme_hues_files($selector, TRUE);
  foreach ($files as $file) {
    $options[$file['fid']] = _theme_hues_user_file_display($file);
  }
  
  $value   = theme_hues_css_value($selector);
  $element = NULL;
  $select  = NULL;
  $file    = NULL;
  $k_base  = 'ruleset-' . $selector['rid'] . '-selector-' . $selector['sid'] . '-' . $property;

  if ($options) {
    $select = array(
      '#type'		        => 'radios',
      '#title'	        => t("Select !item", array('!item' => $property)),
      '#options'	      => $options,
      '#default_value'	=> $value
    );
  }
  
  if ($selector['user_files']) {
    $title = ($options)? t("Or upload your own") : t("Upload !item", array('!item' => $property));
    $file = array(
      '#type'		=> 'file',
      '#title'	=> $title
    );    
  }
  
  if ($select && $file) {
    $element = array(
      '#type'		=> 'fieldset',
      '#title'	=> $property,
      $k_base . '-select' => $select,
      $k_base . '-file'   => $file,
    );
  }
  elseif ($select) {
    $element = $select;
  }
  elseif ($file) {
    $element = $file;
  }
  
  return $element;
}

/**
 * 
 * Display a file value on the select list.
 * 
 * @todo
 * Move to theme()
 * 
 * @param $file
 * 
 * @return
 * Some HTML
 */
function _theme_hues_user_file_display($file) {
  return $file['file']['filename'];
}

/**
 * 
 * Validator for the Theme Hues configuration form for the end user.
 * 
 * @todo
 * Validate values against CSS specs
 */
function theme_hues_configure_validate(&$form, &$form_state) {
}

/**
 * 
 * Submit handler for the Theme Hues configuration form for the end user.
 */
function theme_hues_configure_submit($form, &$form_state) {
//  drupal_set_message("<pre>" . print_r($form_state['values'], TRUE) . "</pre>");
  $rulesets = theme_hues_rulesets();
  if (!$rulesets) {
    drupal_set_message(t("WARNING: no rulesets found during form submission."), 'warning');
    return;
  }
  
  foreach ($rulesets as $ruleset) {
    $k_base = 'ruleset-' . $ruleset['rid'];
    foreach ($ruleset['selectors'] as $selector) {
      $k_selector = $k_base . '-selector-' . $selector['sid'];
      $k_value    = $k_selector . '-value';
      
      // this won't pick up file or multiple choice elements (for now, just images)
      $value = $form_state['values'][$k_value];
      if (!$value) {
        // font family is not implemented as a file choice in the UI but 
        // we're leaving this here for when it will be.
        foreach (array('image', 'font') as $property) {
          $k_select = $k_selector . '-' . $property . '-select';
          $k_file   = $k_selector . '-' . $property . '-file';
          $path     = ($property == 'image')? theme_hues_image_path() : theme_hues_font_path();
          
          // file takes precedence
          $file = file_save_upload($k_file, NULL, $path);
          if ($file) {
            $selector['files'] = array(
              $file->fid => array(
                'fid'     => $file->fid,
                'global'	=> FALSE,
                'file'		=> $file          
              )
            );
            
            _theme_hues_store_selector_files($selector);
            $value = $file->fid;
            break;
          }
          else {
            // check for a select choice
            $value = (isset($form_state['values'][$k_select]))? $form_state['values'][$k_select] : NULL;
            break;
          }
        }        
      }
      
      // no value, no uploaded file, not interested
      if (!$value) {
        continue;
      }
      
      $instance = array(
        'selector'	=> $selector,
        'sid'	      => $selector['sid'],
        'value'	    => $value
      );
      
      $replace_all = ($ruleset['restrictions']['user']['condition'] != 'uid');
      theme_hues_save_instance($instance, $replace_all);
    }
  }
  
  if (theme_hues_css_generate_files()) {
    drupal_set_message(t("Your settings have been saved.  You may need to clear your browser's cache in order to see your new theme settings."));
  }
}